﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net;

using System.IO;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json;
using System.Collections.Specialized;

namespace PrizmDocConvertHTMLtoPDF
{
    class PrizmDocConvert
    {
        private string _apiKey;

        //start our class and make sure that the apikey exists
        public PrizmDocConvert(string apiKey)
        {
            if (String.IsNullOrEmpty(apiKey))
            {
                throw new ArgumentException("Missing parameter apiKey", "apiKey");
            }

            _apiKey = apiKey;
        }

        /*
         * ConvertStatus
         * 
         * Parameters:
         * 
         * affinityToken:  String.The affinity token returned in the Work File API upload.
         * fileID:  String.The FileId returned in the Work File API upload.
         * 
         * Returns:  JOjbect of PrizmDoc Content Conversion API
         * 
         * */


        public async Task<JObject> ConvertStatus(string processId, string affinityToken)
        {
            ///v2/contentConverters/{processId}
            string endpoint = "https://api.accusoft.com/v2/contentConverters/" + processId;
            
            using (var client = new WebClient())
            {
                /*
                 * Required Headers:
                 * Content-Type:  should be application/json
                 * Accusoft-Affinity-Token:  Affinity token returned in post response body for work file specified by fileId parameter in request body.
                 * acs-api-key
                 * 
                 * */
                client.BaseAddress = endpoint;
                client.Headers.Add("acs-api-key", _apiKey);
                client.Headers.Add("Accusoft-Affinity-Token", affinityToken);
                //client.QueryString.Add("processId", processId);
                //Console.WriteLine(client.QueryString.AllKeys.ToString());

                //Console.Write("Resulting Request Headers: ");
                //Console.WriteLine(client.Headers.ToString());


                //nouploading required

                string results = await client.DownloadStringTaskAsync(endpoint);
                
                return JObject.Parse(results);


            }
        }


        /*
         * Using the Content Converstion API, POST a JSON file based on the fileID and affinity token
         * that was generated by the Work File API.
         * 
         * Parameters:
         * 
         * affinityToken:  String.  The affinity token returned in the Work File API upload.
         * fileID:  String.  The FileId returned in the Work File API upload.
         * format:  The format to convert to, defaults to PDF.
         * 
         * Returns:  JOjbect of PrizmDoc Content Conversion API
         * 
         * */
        public async Task<JObject> Convert(string affinityToken, string fileID, string format="pdf")
        {
            var endpoint = new Uri("https://api.accusoft.com/v2/contentConverters");

            using (var client = new WebClient())
            {
                /*
                 * Required Headers:
                 * Content-Type:  should be application/json
                 * Accusoft-Affinity-Token:  Affinity token returned in post response body for work file specified by fileId parameter in request body.
                 * acs-api-key
                 * 
                 * */
                client.Headers.Add("Content-Type", "application/json");
                client.Headers.Add("acs-api-key", _apiKey);
                client.Headers.Add("Accusoft-Affinity-Token", affinityToken);

                //create the JSON object
                //create the JSON object
                JObject tester =
                new JObject(
                    new JProperty("input",
                        new JObject(
                            new JProperty("sources",
                                new JArray(
                                    new JObject(
                                        new JProperty("fileId", fileID)
                                    )
                                )
                            ),
                            new JProperty("dest",
                                new JObject(
                                    new JProperty("format", format)
                                )
                            )
                        )
                    )
                );

                //now send up the file
                //we get back a JSON string
                string results = await client.UploadStringTaskAsync(endpoint, "POST", tester.ToString());

                return JObject.Parse(results);


            }

        }

        /*
         * DownloadWork File
         * Parameters:
         * 
         * affinityToken:  The affinityToken from the originally uploaded Work File API upload
         * fileId:  The fileId of the file to be downloaded
         * outfile:  The file name to save the Work File API downloaded file to.
         * 
         * 
         * Returns:  Nothing
         */



        public async Task DownloadWorkfile(string affinityToken, string fileId, string outfile)
        {
            var endpoint = new Uri("https://api.accusoft.com/PCCIS/V1/WorkFile/" + fileId);

            using (var client = new WebClient())
            {
                /*
                 * Required Headers:
                 * Content-Type:  should be application/json
                 * Accusoft-Affinity-Token:  Affinity token rC:\Users\johnhummel\documents\visual studio 2017\Projects\PrizmDocConvertHTMLtoPDF\PrizmDocConvertHTMLtoPDF\HTMLConvert.cseturned in post response body for work file specified by fileId parameter in request body.
                 * acs-api-key
                 * 
                 * */
                client.Headers.Add("acs-api-key", _apiKey);
                client.Headers.Add("Accusoft-Affinity-Token", affinityToken);


                //now, pipe those results back out to a file
                FileInfo output = new FileInfo(outfile);

                using (var writeStream = output.Create())
                {
                    var results = await client.DownloadDataTaskAsync(endpoint);
                    await writeStream.WriteAsync(results, 0, results.Length);
                }

            }
        }

        /*
         * Adding a file with WorkFile API
         * 
         * Parameters:
         * 
         * fileToUpload:  The file name to be uploaded
         * 
         * Returns:  JOjbect of PrizmDoc Work File API
         * 
         * 
         * */

        public async Task<JObject> Upload(string fileToUpload)
        {
            FileInfo input = new FileInfo(fileToUpload);


            //check that the references aren't null
            if (input == null)
            {
                throw new ArgumentException("Missing parameter input", "input");
            }

            var fileName = input.Name;
            var endpoint = new Uri("https://api.accusoft.com/PCCIS/V1/WorkFile");
            //Console.WriteLine(endpoint.ToString());

            using (var client = new WebClient())
            {
                /*
                 * Required Headers:
                 * acs-api-key
                 * Content-Type - should be application/octet-stream
                 * */
                client.Headers.Add("acs-api-key", _apiKey);
                client.Headers.Add("Content-Type", "application/octet-stream");

                using (var reader = new BinaryReader(input.OpenRead()))
                {
                    var data = reader.ReadBytes((int)reader.BaseStream.Length);
                    var results = await client.UploadDataTaskAsync(endpoint, "POST", data);

                    string getResult = "";
                    getResult = Encoding.ASCII.GetString(results);

                    return JObject.Parse(getResult);


                }
            }
        }
    }
}
